use serde_json::json;
use serde_json::Value;
use std::process::{Stdio, Command};

use crate::common;
use crate::bar_modules;

#[derive(Clone)]
pub struct UnibarModuleNetwork {
    opts:           common::AppOptions,
    ip_addr_stdout: String,
    network_info:   Value,
}

impl UnibarModuleNetwork {

    // --------------------
    pub fn new(o :common::AppOptions) -> Self {
        UnibarModuleNetwork {
            opts: o,
            ip_addr_stdout: "".to_string(),
            network_info: json!(serde_json::Value::Null),
        }
    }
}

impl bar_modules::BarModuleActions for UnibarModuleNetwork {

    // --------------------
    fn clear(&mut self) {
        self.network_info = json!(serde_json::Value::Null);
    }

    // --------------------
    fn generate_data(&mut self) {
        // Output of 'ip -j address' command has network information
        match Command::new("ip")
                      .arg("-j") // Output in json format
                      .arg("address")
                      .stdout(Stdio::piped())
                      .output() {
            Err(e) => {
                eprintln!("Error getting network information {e}");
                self.network_info = json!(serde_json::Value::Null);
            }
            Ok(ip_addr_output) => {
                self.ip_addr_stdout = String::from_utf8(ip_addr_output.stdout).unwrap();
                self.ip_addr_stdout.pop();

                let network_data: Value = serde_json::from_str::<Value>(self.ip_addr_stdout.as_str()).unwrap();

                if let Some(interfaces) = network_data.as_array() {
                    if self.opts.debug {
                        println!("-----> network_data - {:#?}", network_data);
                    }
                    // Get all interfaces that are up
                    let mut up_intf :Vec<_> = interfaces.iter()
                                            .filter(|el| el["operstate"].as_str().unwrap().contains("UP"))
                                            .cloned().collect();

                    // Sort by 'ifname'. This is an unreliable way to proiritize ethernet over wifi.
                    // Ethenet network interface names normally start with 'e' and wifi interface names
                    // start with 'w'
                    up_intf.sort_by(|a, b| a["ifname"].as_str().unwrap().cmp(b["ifname"].as_str().unwrap()));
                    if up_intf.len() > 0 {
                        let inet_addr :Vec<_> = up_intf[0]["addr_info"].as_array().unwrap().iter()
                                                .filter(|ai| ai["scope"].as_str().unwrap().contains("global"))
                                                .cloned().collect();
                        if inet_addr.len() > 0 {
                            if self.opts.debug {
                                println!("-----> Inet Addr - {:#?}", inet_addr);
                            }
                            self.network_info = inet_addr[0].clone();
                            self.network_info["ifname"] = up_intf[0]["ifname"].clone();
                        }
                    }
                }
            }
        }
        if self.opts.debug {
            println!("-----> ip_addr - {:#?}", self.network_info);
        }
    }

    // --------------------
    fn get_content(&self) -> String {
        if self.network_info != serde_json::Value::Null {
            // If any interface was up, return the local IP address
            return self.network_info["local"].as_str().unwrap().to_string();
        }
        return "Network down".to_string();
    }

    // --------------------
    fn get_icon(&self) -> String {
        if self.network_info != serde_json::Value::Null {
            // Select icon based on which interface is up (ethernet or wifi)
            if self.network_info["ifname"].as_str().unwrap().to_string().starts_with("w") {
                return "📶".to_string();
            } else {
                return "🌎".to_string();
            }
        }
        return "📡".to_string();
    }
}

impl bar_modules::BarModuleDebug for UnibarModuleNetwork {

    // --------------------
    fn post_debug(&self) {
    }
}
